@@ -27,12 +27,11 @@ tvFE <- function(x, ...) UseMethod("tvFE", x)
 #' @param obs A scalar with the number of time observations
 #' @param est The nonparametric estimation method, one of "lc" (default) for linear constant
 #'  or "ll" for local linear.
-#' @param tkernel The type of kernel used in the coefficients estimation method,
-#' one of Epanesnikov ("Epa") or "Gaussian".
+#' @param tkernel A character, either "Triweight" (default), "Epa" or "Gaussian" kernel function.
 #
 #' @export
 tvFE.matrix<-function(x, y, z = NULL, ez = NULL, bw, neq, obs,
-               est = c("lc", "ll"), tkernel = c("Epa", "Gaussian"), ...)
+               est = c("lc", "ll"), tkernel = c("Triweight", "Epa", "Gaussian"), ...)
 {
   x <- as.matrix(x)
   y <- as.numeric(y)
@@ -44,7 +43,7 @@ tvFE.matrix<-function(x, y, z = NULL, ez = NULL, bw, neq, obs,
   if(!is.null(z))
   {
     if(length(z) != obs)
-      stop("\nDimensions of 'x' and 'z' are not compatible\n")
+      stop("\nDimensions of 'x' and 'z' are not compatible. \nThe size of 'z' should be 'obs'.\n")
     grid <- z
   }
   else
@@ -58,23 +57,24 @@ tvFE.matrix<-function(x, y, z = NULL, ez = NULL, bw, neq, obs,
   fitted = resid <- numeric(neq*eobs)
   theta <- matrix(0, eobs, nvar)
   alpha <- matrix(0, nrow = eobs, ncol = neq-1)
-  D <- t(cbind(rep(-1, neq-1), diag(1, neq-1)))%x%rep(1, obs)
   for (t in 1:eobs)
   {
-    tau0 <- grid - grid[t]
-    xtemp <- x
-    if(est == "ll")
-      xtemp <- cbind (xtemp, xtemp * tau0)
+    tau0 <- grid - ez[t]
     kernel.bw <- .kernel(tau0, bw, tkernel)
     if (length (kernel.bw != 0) < 3)
       stop("Bandwidth is too small.\n")
+    D <- t(cbind(rep(-1, neq-1), diag(1, neq-1)))%x%rep(1, obs)
+    xtemp <- x
+    if(est == "ll")
+      xtemp <- cbind (xtemp, xtemp * tau0)
     WH <- diag(neq)%x%diag(kernel.bw)
-    temp <- crossprod(D, WH)
-    MH <- diag(neq*obs) - D %*% solve(temp%*%D)%*%temp
+    DW <- crossprod(D, WH)
+    temp <- qr.solve(DW%*%D)%*%DW
+    MH <- diag(neq*obs) - D %*% temp
     SH <- crossprod(MH, WH)%*%MH
     x.tilde <- crossprod(xtemp, SH)
-    s0 <- x.tilde %*% xtemp
     T0 <- x.tilde %*% y
+    s0 <- x.tilde %*% xtemp
     result <- try(qr.solve(s0, T0), silent = TRUE)
     if(inherits(result, "try-error"))
       result <- try(qr.solve(s0, T0, tol = .Machine$double.xmin ), silent = TRUE)
@@ -83,15 +83,19 @@ tvFE.matrix<-function(x, y, z = NULL, ez = NULL, bw, neq, obs,
            Possibly, the 'bw' is too small for values in 'ez'.\n")
     theta[t, ] <- result[1:nvar]
     xtheta <- x%*%theta[t,]
-    temp3 <- crossprod(D, WH)
-    temp4 <- temp3%*%D
-    alpha[t,] <- as.numeric(solve(temp4)%*%temp3%*%(y - xtheta))
-    fitted[t+((1:neq)-1)*eobs] <- xtheta[t+((1:neq)-1)*eobs]
+    alpha[t,] <- as.numeric(temp%*%(y - xtheta))
+    fitted[t+((1:neq)-1)*eobs] <- xtheta[t+((1:neq)-1)*obs]
   }
   alpha <- apply(alpha, 2, mean)
-  fitted <- fitted + D %*% alpha
   if(!is.predict)
+  {
     resid <- y - fitted 
+    D <- Matrix(t(cbind(rep(-1, neq-1), diag(1, neq-1)))%x%rep(1, eobs))
+    fitted <- drop(fitted + D %*% alpha)
+  }
+  else
+    fitted = resid <- NULL
+    
   return(list(coefficients = theta, fitted = fitted, residuals = resid, alpha = c(-sum(alpha), alpha)))
 }
 
@@ -108,7 +112,7 @@ tvFE.tvplm <- function(x, ...)
 #' @keywords internal
 .tvFE.cv<-function(bw, x, y, z = NULL, neq, obs, cv.block = 0,
                    est = c("lc", "ll"), 
-                   tkernel = c("Epa", "Gaussian"))
+                   tkernel = c("Triweight", "Epa", "Gaussian"))
 {
   x <- as.matrix(x)
   fitted = resid <- numeric(obs*neq)
@@ -117,25 +121,26 @@ tvFE.tvplm <- function(x, ...)
   if(!is.null(z))
   {
     if(length(z) != obs)
-      stop("\nDimensions of 'x' and 'z' are not compatible\n")
+      stop("\nDimensions of 'x' and 'z' are not compatible. \nThe size of 'z' should be 'obs'.\n")
     grid <- z
   }
   else
     grid <- (1:obs)/obs
-  D <- t(cbind(rep(-1, neq-1), diag(1, neq-1)))%x%rep(1, obs)
   for (t in 1:obs)
   {
     tau0 <- grid - grid[t]
-    xtemp <- x
-    if(est == "ll")
-      xtemp <- cbind (xtemp, xtemp * tau0)
     kernel.bw <- .kernel(tau0, bw, tkernel)
     kernel.bw[max(1, t-cv.block):min(t+cv.block, obs)] <- 0
     if (sum(kernel.bw != 0) < 3) 
       return (.Machine$double.xmax)
+    D <- t(cbind(rep(-1, neq-1), diag(1, neq-1)))%x%rep(1, obs)
+    xtemp <- x
+    if(est == "ll")
+      xtemp <- cbind (xtemp, xtemp * tau0)
     WH <- diag(neq)%x%diag(kernel.bw)
-    temp <- crossprod(D, WH)
-    MH <- diag(neq*obs) - D %*% solve(temp%*%D)%*%temp
+    DW <- crossprod(D, WH)
+    temp <- qr.solve(DW%*%D)%*%DW
+    MH <- diag(neq*obs) - D %*% temp
     SH <- crossprod(MH, WH)%*%MH
     x.tilde <- crossprod(xtemp, SH)
     T0 <- x.tilde %*% y
@@ -143,13 +148,12 @@ tvFE.tvplm <- function(x, ...)
     result <- try(qr.solve(s0, T0), silent = TRUE)
     if(inherits(result, "try-error"))
       return (.Machine$double.xmax)
-    xtheta <- x%*%result[1:nvar]
+    xtheta <- x%*%matrix(result[1:nvar])
+    alpha[t,] <- as.numeric(temp%*%(y - xtheta))
     fitted[t+((1:neq)-1)*obs] <- xtheta[t+((1:neq)-1)*obs]
-    temp3 <- crossprod(D, WH)
-    temp4 <- temp3%*%D
-    alpha[t,] <- as.numeric(solve(temp4)%*%temp3%*%(y - xtheta))
   }
   alpha <- apply(alpha, 2, mean)
+  D <- t(cbind(rep(-1, neq-1), diag(1, neq-1)))%x%rep(1, obs)
   resid <- y - fitted - D%*%alpha
   return(mean(resid^2))
 }
